SystemOrganization addCategory: #MkPathTrainer!
SystemOrganization addCategory: #'MkPathTrainer-Tests'!

TestCase subclass: #PTFileReferencingProblemGeneratorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MkPathTrainer-Tests'!

!PTFileReferencingProblemGeneratorTest methodsFor: 'tests' stamp: 'MathiasKegelmann 12/9/2011 18:04'!
testTargetFileIn
	"Tests PTFileReferencingProblemGenerator>>#targetFileIn:"
	| pg |
	pg := PTFileReferencingProblemGenerator new.
	self assert: '/a/b/b.html' equals: (pg targetFileIn: '/a/b').
	self assert: '/a/Termine/termine.html' equals: (pg targetFileIn: '/a/Termine').
	self assert: '/a/bilder/ausflug.jpg' equals: (pg targetFileIn: '/a/bilder').
	self assert: '/a/Bilder/ausflug.jpg' equals: (pg targetFileIn: '/a/Bilder')! !

TestCase subclass: #PTFileSystemTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MkPathTrainer-Tests'!

!PTFileSystemTest methodsFor: 'tests' stamp: 'MathiasKegelmann 12/6/2011 20:06'!
assert: expected equalsAsBag: actual
	self assert: expected asBag equals: actual asBag! !

!PTFileSystemTest methodsFor: 'tests' stamp: 'MathiasKegelmann 12/4/2011 22:30'!
testEmptyTree
	self assert: { '/' } equals: PTFileSystem new allPaths.! !

!PTFileSystemTest methodsFor: 'tests' stamp: 'MathiasKegelmann 12/4/2011 22:32'!
testEmptyTreeContainsRoot
	self assert: (PTFileSystem new contains: '/')! !

!PTFileSystemTest methodsFor: 'tests' stamp: 'MathiasKegelmann 12/6/2011 20:08'!
testForkingTree
	| fileSystem |
	fileSystem := PTFileSystem withAll: #('/a/b' '/a/c' 'a/d/e').
	self assert: #('/' '/a' '/a/b' '/a/c' '/a/d' '/a/d/e') equalsAsBag: fileSystem allPaths.! !

!PTFileSystemTest methodsFor: 'tests' stamp: 'MathiasKegelmann 12/6/2011 21:20'!
testForkingTreeMaximalPaths
	| fileSystem |
	fileSystem := PTFileSystem withAll: #('/a/b' '/a/c' 'a/d/e').
	self assert: #('/a/b' '/a/c' '/a/d/e') equalsAsBag: fileSystem maximalPaths! !

!PTFileSystemTest methodsFor: 'tests' stamp: 'MathiasKegelmann 12/6/2011 20:05'!
testSimpleTree
	| fileSystem |
	fileSystem := PTFileSystem with: '/hello/world'.
	self assert: { '/' . '/hello' . '/hello/world' } equalsAsBag: fileSystem allPaths.! !

TestCase subclass: #PTRelativePathFinderTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MkPathTrainer-Tests'!

!PTRelativePathFinderTest methodsFor: 'as yet unclassified' stamp: 'MathiasKegelmann 12/14/2011 20:33'!
testDot
	| pf |
	pf := PTRelativePathFinder from: '/hello/world' to: '/hello/world'.
	self assert: '.' equals: pf findPath! !

!PTRelativePathFinderTest methodsFor: 'as yet unclassified' stamp: 'MathiasKegelmann 12/14/2011 20:37'!
testDoubleStep
	| pf |
	pf := PTRelativePathFinder from: '/hello' to: '/hello/world/done'.
	self assert: 'world/done' equals: pf findPath! !

!PTRelativePathFinderTest methodsFor: 'as yet unclassified' stamp: 'MathiasKegelmann 12/14/2011 20:37'!
testDoubleStepFromRoot
	| pf |
	pf := PTRelativePathFinder from: '/' to: '/hello/world'.
	self assert: 'hello/world' equals: pf findPath! !

!PTRelativePathFinderTest methodsFor: 'as yet unclassified' stamp: 'MathiasKegelmann 12/14/2011 21:01'!
testDoubleStepUp
	| pf |
	pf := PTRelativePathFinder from: '/hello/world/x' to: '/hello'.
	self assert: '../..' equals: pf findPath! !

!PTRelativePathFinderTest methodsFor: 'as yet unclassified' stamp: 'MathiasKegelmann 12/14/2011 21:01'!
testDoubleStepUpThenDown
	| pf |
	pf := PTRelativePathFinder from: '/hello/world/x' to: '/hello/a/b/c'.
	self assert: '../../a/b/c' equals: pf findPath! !

!PTRelativePathFinderTest methodsFor: 'as yet unclassified' stamp: 'MathiasKegelmann 12/15/2011 22:17'!
testDoubleStepUpThenDownSteps
	| pf |
	pf := PTRelativePathFinder from: '/hello/world/x' to: '/hello/a/b/c'.
	self assert: (OrderedCollection with: '..' with: '..' with: 'a' with: 'b' with: 'c')
		equals: pf findSteps! !

!PTRelativePathFinderTest methodsFor: 'as yet unclassified' stamp: 'MathiasKegelmann 12/14/2011 21:01'!
testDoubleStepUpToRoot
	| pf |
	pf := PTRelativePathFinder from: '/hello/world' to: '/'.
	self assert: '../..' equals: pf findPath! !

!PTRelativePathFinderTest methodsFor: 'as yet unclassified' stamp: 'MathiasKegelmann 12/14/2011 20:33'!
testSingleStep
	| pf |
	pf := PTRelativePathFinder from: '/hello' to: '/hello/world'.
	self assert: 'world' equals: pf findPath! !

!PTRelativePathFinderTest methodsFor: 'as yet unclassified' stamp: 'MathiasKegelmann 12/14/2011 20:38'!
testSingleStepUp
	| pf |
	pf := PTRelativePathFinder from: '/hello/world' to: '/hello'.
	self assert: '..' equals: pf findPath! !

!PTRelativePathFinderTest methodsFor: 'as yet unclassified' stamp: 'MathiasKegelmann 12/14/2011 21:02'!
testStepUpToRootThenDown
	| pf |
	pf := PTRelativePathFinder from: '/hello/world/x' to: '/a/b/c'.
	self assert: '../../../a/b/c' equals: pf findPath! !

TestCase subclass: #PTRelativePathWalkerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MkPathTrainer-Tests'!

!PTRelativePathWalkerTest methodsFor: 'tests' stamp: 'MathiasKegelmann 12/7/2011 21:25'!
testAboveRoot
	| walker |
	walker := PTRelativePathWalker startingAt: '/a'.
	self should: [ walker walkPath: '../..' ] raise: Error.
! !

!PTRelativePathWalkerTest methodsFor: 'tests' stamp: 'MathiasKegelmann 12/7/2011 18:22'!
testAbsolutePathSignalsException
	| walker |
	walker := PTRelativePathWalker startingAt: ''.
	self should: [ walker walkPath: '/a' ] raise: Error! !

!PTRelativePathWalkerTest methodsFor: 'tests' stamp: 'MathiasKegelmann 12/7/2011 21:28'!
testComplexPath
	| walker |
	walker := PTRelativePathWalker startingAt: '/a/b'.
	walker walkPath: './../c/d/./e/../f'.
	self assert: '/a/c/d/f' equals: walker currentPath.! !

!PTRelativePathWalkerTest methodsFor: 'tests' stamp: 'MathiasKegelmann 12/7/2011 08:13'!
testDoubleStep
	| walker |
	walker := PTRelativePathWalker startingAt: '/a/b'.
	walker walkPath: 'hello/world'.
	self assert: '/a/b/hello/world' equals: walker currentPath.! !

!PTRelativePathWalkerTest methodsFor: 'tests' stamp: 'MathiasKegelmann 12/7/2011 08:14'!
testDoubleStepFromEmptyStringAsRoot
	| walker |
	walker := PTRelativePathWalker startingAt: ''.
	walker walkPath: 'hello/world'.
	self assert: '/hello/world' equals: walker currentPath.! !

!PTRelativePathWalkerTest methodsFor: 'tests' stamp: 'MathiasKegelmann 12/7/2011 08:14'!
testDoubleStepFromRoot
	| walker |
	walker := PTRelativePathWalker startingAt: '/'.
	walker walkPath: 'hello/world'.
	self assert: '/hello/world' equals: walker currentPath.! !

!PTRelativePathWalkerTest methodsFor: 'tests' stamp: 'MathiasKegelmann 12/7/2011 08:00'!
testSingleStep
	| walker |
	walker := PTRelativePathWalker startingAt: '/a/b'.
	walker walkPath: 'test'.
	self assert: '/a/b/test' equals: walker currentPath.! !

!PTRelativePathWalkerTest methodsFor: 'tests' stamp: 'MathiasKegelmann 12/7/2011 21:26'!
testStay
	| walker |
	walker := PTRelativePathWalker startingAt: '/a/b'.
	walker walkPath: '.'.
	self assert: '/a/b' equals: walker currentPath.! !

!PTRelativePathWalkerTest methodsFor: 'tests' stamp: 'MathiasKegelmann 12/7/2011 18:31'!
testTwoDownOneUp
	| walker |
	walker := PTRelativePathWalker startingAt: '/a/b'.
	walker walkPath: 'c/d'.
	walker walkPath: '..'.
	self assert: '/a/b/c' equals: walker currentPath.! !

!PTRelativePathWalkerTest methodsFor: 'tests' stamp: 'MathiasKegelmann 12/7/2011 08:14'!
testTwoSteps
	| walker |
	walker := PTRelativePathWalker startingAt: '/a/b'.
	walker walkPath: 'hello'.
	walker walkPath: 'world'.
	self assert: '/a/b/hello/world' equals: walker currentPath.! !

!PTRelativePathWalkerTest methodsFor: 'tests' stamp: 'MathiasKegelmann 12/7/2011 21:22'!
testTwoUp
	| walker |
	walker := PTRelativePathWalker startingAt: '/a/b/c'.
	walker walkPath: '../..'.
	self assert: '/a' equals: walker currentPath.! !

!PTRelativePathWalkerTest methodsFor: 'tests' stamp: 'MathiasKegelmann 12/7/2011 18:29'!
testUpStep
	| walker |
	walker := PTRelativePathWalker startingAt: '/a/b'.
	walker walkPath: '..'.
	self assert: '/a' equals: walker currentPath.! !

!PTRelativePathWalkerTest methodsFor: 'tests' stamp: 'MathiasKegelmann 12/7/2011 21:23'!
testUpToRoot
	| walker |
	walker := PTRelativePathWalker startingAt: '/a/b'.
	walker walkPath: '../..'.
	self assert: '/' equals: walker currentPath.! !

TestCase subclass: #PTSlidingWindowCounterTest
	instanceVariableNames: 'counter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MkPathTrainer-Tests'!

!PTSlidingWindowCounterTest methodsFor: 'as yet unclassified' stamp: 'MathiasKegelmann 12/9/2011 07:53'!
setUp
	counter := PTSlidingWindowCounter new: 3 allowedErrors: 1! !

!PTSlidingWindowCounterTest methodsFor: 'as yet unclassified' stamp: 'MathiasKegelmann 12/9/2011 07:51'!
testInitiallyFalse
	self deny: counter value! !

!PTSlidingWindowCounterTest methodsFor: 'as yet unclassified' stamp: 'MathiasKegelmann 12/9/2011 08:15'!
testNYNY
	counter add: false; add: true; add: false; add: true.
	self assert: counter value! !

!PTSlidingWindowCounterTest methodsFor: 'as yet unclassified' stamp: 'MathiasKegelmann 12/9/2011 08:10'!
testNYY
	counter add: false; add: true; add: true.
	self assert: counter value! !

!PTSlidingWindowCounterTest methodsFor: 'as yet unclassified' stamp: 'MathiasKegelmann 12/9/2011 08:15'!
testYNNY
	counter add: true; add: false; add: false; add: true.
	self deny: counter value! !

!PTSlidingWindowCounterTest methodsFor: 'as yet unclassified' stamp: 'MathiasKegelmann 12/9/2011 08:12'!
testYNY
	counter add: true; add: false; add: true.
	self assert: counter value! !

!PTSlidingWindowCounterTest methodsFor: 'as yet unclassified' stamp: 'MathiasKegelmann 12/9/2011 08:10'!
testYY
	counter add: true; add: true.
	self deny: counter value! !

!PTSlidingWindowCounterTest methodsFor: 'as yet unclassified' stamp: 'MathiasKegelmann 12/9/2011 08:11'!
testYYN
	counter add: true; add: true; add: false.
	self deny: counter value! !

!PTSlidingWindowCounterTest methodsFor: 'as yet unclassified' stamp: 'MathiasKegelmann 12/9/2011 08:14'!
testYYNY
	counter add: true; add: true; add: false; add: true.
	self assert: counter value! !

!PTSlidingWindowCounterTest methodsFor: 'as yet unclassified' stamp: 'MathiasKegelmann 12/9/2011 08:06'!
testYYY
	counter add: true; add: true; add: true.
	self assert: counter value! !

!PTSlidingWindowCounterTest methodsFor: 'as yet unclassified' stamp: 'MathiasKegelmann 12/9/2011 08:14'!
testYYYN
	counter add: true; add: true; add: true; add: false.
	self deny: counter value! !

!PTSlidingWindowCounterTest methodsFor: 'as yet unclassified' stamp: 'MathiasKegelmann 12/9/2011 08:13'!
testYYYY
	counter add: true; add: true; add: true; add: true.
	self assert: counter value! !

WAFileLibrary subclass: #PTFileLibrary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MkPathTrainer'!

!PTFileLibrary methodsFor: 'accessing' stamp: 'MathiasKegelmann 12/10/2011 12:02'!
mainCss
	^ '
body {
  font-family: "Droid Sans", sans-serif;
  padding-left: 30px; padding-right: 30px; padding-top: 30px;
  min-width: 30em; max-width: 50em;
  margin-left: auto; margin-right: auto;
  background-color: white;
}
div.main {
  padding: 2ex;
  background-color: rgb(220, 220, 255);
  color: #black;
  border: 2px solid blue;
  border-radius: 8px; -moz-border-radius: 8px; -webkit-border-radius: 8px;
}
h1 { 
  font-family: Tangerine, cursive;
  font-weight: 700;
  font-size: 100px;
  position: relative;
  left: -50px;
  color: rgb(0, 0, 100); margin-top: 0px; 
  margin: 0px
}
span.path {
  color: blue;
  font-family: "Droid Sans Mono", monospace;
}
input { border-radius: 4px; -moz-border-radius: 4px; -webkit-border-radius: 4px; margin-top: 2ex; }
.footer { color: gray; font-style: italic; font-size: x-small; }
'! !

WAComponent subclass: #PTRootComponent
	instanceVariableNames: 'problemGenerator answerPath generatorQueue answerCorrectTheFirstTime'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MkPathTrainer'!

!PTRootComponent class methodsFor: 'testing' stamp: 'MathiasKegelmann 12/6/2011 21:29'!
canBeRoot
	^ true! !

!PTRootComponent class methodsFor: 'class initialization' stamp: 'MathiasKegelmann 12/6/2011 21:35'!
initialize
	(WAAdmin register: self asApplicationAt: 'pfad')
		addLibrary: JQDeploymentLibrary ! !

!PTRootComponent methodsFor: 'accessing' stamp: 'MathiasKegelmann 12/7/2011 21:30'!
answerPath
	^ answerPath! !

!PTRootComponent methodsFor: 'accessing' stamp: 'MathiasKegelmann 12/7/2011 21:30'!
answerPath: anObject
	answerPath := anObject! !

!PTRootComponent methodsFor: 'initialization' stamp: 'MathiasKegelmann 12/9/2011 08:29'!
createFileSystem
	^ PTFileSystem withAll: #(
		'/html/ilvesheimmlk/Neues_Kinderhaus/Bilder'
		'/html/ilvesheimmlk/Neues_Kinderhaus/Termine'
		'/html/ilvesheimmlk/Neues_Kinderhaus/Mitarbeiter'
		'/html/ilvesheimmlk/bilder/Kindergarten'
		'/html/ilvesheimmlk/bilder/Kindergottesdienst'
		'/html/ilvesheimmlk/bilder/Papafest'
		'/html/ilvesheimmlk/aktuelles'
		'/html/ilvesheimmlk/halligalli'
		'/html/ilvesheimmlk/kirchenchor'
		'/html/ilvesheimmlk/termine'
		'/html/ilvesheimmlk/mitarbeiter'
		'/html/Termine'
	)
	! !

!PTRootComponent methodsFor: 'initialization' stamp: 'MathiasKegelmann 12/9/2011 18:14'!
createGeneratorQueueFor: aFileSystem
	^ (Array 
		with: (PTNonBranchingProblemGenerator for: aFileSystem)
		with: (PTBranchingProblemGenerator for: aFileSystem)
		with: (PTFileReferencingProblemGenerator for: aFileSystem)
	) asOrderedCollection ! !

!PTRootComponent methodsFor: 'accessing' stamp: 'MathiasKegelmann 12/8/2011 18:40'!
falseMessage
	^ 'Falsch!!', 
		String cr, 
		([ 'Die Antwort fhrt zu ', self pathForAnswer, '.' ]
			on: Error do: [ 'Illegaler Pfad!!' ])! !

!PTRootComponent methodsFor: 'initialization' stamp: 'MathiasKegelmann 12/9/2011 10:06'!
initialize
	super initialize.
	self reset! !

!PTRootComponent methodsFor: 'accessing' stamp: 'MathiasKegelmann 12/10/2011 12:09'!
isAnswerCorrect
	"evaluate the answer and remember whether we got it right the first time"
	| correct |
	correct := 
		[ problemGenerator target = self pathForAnswer ] 
			on: Error do: [ false ].
	answerCorrectTheFirstTime := answerCorrectTheFirstTime and: [ correct ].
	^correct! !

!PTRootComponent methodsFor: 'action' stamp: 'MathiasKegelmann 12/10/2011 12:08'!
nextProblem
	"count the result in the problem generator and produce the next problem"
      problemGenerator count: answerCorrectTheFirstTime.
	self resetAnswer.
	problemGenerator isSufficient 
		ifTrue: [ self nextProblemGenerator ]
		ifFalse:	[ problemGenerator nextProblem ]
! !

!PTRootComponent methodsFor: 'action' stamp: 'MathiasKegelmann 12/9/2011 08:30'!
nextProblemGenerator
	problemGenerator :=
		generatorQueue 
			ifEmpty: [ nil ]
			ifNotEmpty: [ generatorQueue removeFirst ]! !

!PTRootComponent methodsFor: 'accessing' stamp: 'MathiasKegelmann 12/8/2011 18:39'!
pathForAnswer
	"may signal an error for illegal paths"
	^ ((PTRelativePathWalker startingAt: problemGenerator source) 
			walkPath: answerPath trimBoth) currentPath! !

!PTRootComponent methodsFor: 'rendering' stamp: 'MathiasKegelmann 12/15/2011 18:11'!
renderAnswerFormOn: html
	html form
		onSubmit: (self renderAnswerResultOn: html);
		with: [ 
					html textInput
						size: 50;
						script: (html jQuery new call: #focus);
						on: #answerPath of: self.
					html submitButton: 'OK'.
					html text: ' '.
					html anchor
						url: '#';
						onClick: (self renderAnswerHelpOn: html);
						with: 'Hilfe' ]! !

!PTRootComponent methodsFor: 'rendering-ajax' stamp: 'MathiasKegelmann 12/15/2011 18:16'!
renderAnswerHelpOn: html
	^ (html jQuery: #main) load html: [ :h | self renderHelpOn: h];
		return: false! !

!PTRootComponent methodsFor: 'rendering-ajax' stamp: 'MathiasKegelmann 12/15/2011 18:14'!
renderAnswerResultOn: html
	^ html jQuery getScript
		serializeForm;
		script: [ :s | 
					self isAnswerCorrect
						ifTrue: [ 
							s << s javascript alert: 'Richtig!!'.
							self nextProblem.
							s << (s jQuery: #main) load html: [ :h | self renderMainDivOn: h ] ]
						ifFalse: [ s << s javascript alert: self falseMessage ] ];
		return: false! !

!PTRootComponent methodsFor: 'rendering' stamp: 'MathiasKegelmann 12/9/2011 21:47'!
renderContentOn: html
	html div
		id: #main;
		class: #main;
		with: [ self renderMainDivOn: html ].
	html span
		class: #footer;
		with: 'Version ' , self versionNumber , ', Mathias Kegelmann, ' , self versionDate! !

!PTRootComponent methodsFor: 'rendering' stamp: 'MathiasKegelmann 12/9/2011 10:07'!
renderFinishedOn: html
	html 
		text: 'Geschafft!!';
		paragraph: [
			html anchor
				callback: [ self reset ];
				with: 'Neustart'
		]! !

!PTRootComponent methodsFor: 'rendering' stamp: 'MathiasKegelmann 12/15/2011 22:35'!
renderHelpOn: html
	html heading level: 1; with:  'Hilfe'.
	self renderQuestionOn: html.
	html unorderedList: [ | stepWalker resultWalker |
		resultWalker := PTRelativePathWalker startingAt: problemGenerator source.
		self renderSteps: (PTRelativePathWalker startingAt: '.') to: resultWalker on: html.
		stepWalker := PTRelativePathWalker startingAt: ''.
		(PTRelativePathFinder 
			from: problemGenerator source 
			to: problemGenerator target) 
		findSteps do: [ :step |
			stepWalker add: step.
			resultWalker step: step.
			self renderSteps: stepWalker to: resultWalker on: html ]
		].
	html paragraph: [
		html anchor
			callback: [ answerCorrectTheFirstTime := false ];
			with: 'Zurck' ]! !

!PTRootComponent methodsFor: 'rendering' stamp: 'MathiasKegelmann 12/9/2011 08:40'!
renderMainDivOn: html
	html heading: 'Pfad-Trainer'.
	problemGenerator isNil 
		ifTrue: [ self renderFinishedOn: html ]
		ifFalse: [ self renderQuestionAndAnswerOn: html ]! !

!PTRootComponent methodsFor: 'rendering' stamp: 'MathiasKegelmann 12/9/2011 08:40'!
renderQuestionAndAnswerOn: html
	self renderQuestionOn: html.
	self renderAnswerFormOn: html! !

!PTRootComponent methodsFor: 'rendering' stamp: 'MathiasKegelmann 12/9/2011 21:50'!
renderQuestionOn: html
	problemGenerator renderProblemOn: html
! !

!PTRootComponent methodsFor: 'rendering' stamp: 'MathiasKegelmann 12/15/2011 22:31'!
renderSteps: stepWalker to: resultWalker on: html
	html listItem: [
		html span class: #path; with: stepWalker currentPathWithoutLeadingSlash.
		html text: ' '; encodeCharacter: 8594; text: ' '.
		html span class: #path; with: resultWalker currentPath
		]! !

!PTRootComponent methodsFor: 'initialization' stamp: 'MathiasKegelmann 12/10/2011 12:08'!
reset
	| fileSystem |
	fileSystem := self createFileSystem.
	generatorQueue := self createGeneratorQueueFor: fileSystem.
	self nextProblemGenerator.
	self resetAnswer! !

!PTRootComponent methodsFor: 'action' stamp: 'MathiasKegelmann 12/10/2011 12:08'!
resetAnswer
	answerCorrectTheFirstTime := true.
	self answerPath: ''! !

!PTRootComponent methodsFor: 'updating' stamp: 'MathiasKegelmann 12/10/2011 12:02'!
updateRoot: anHtmlRoot
	super updateRoot: anHtmlRoot.
	anHtmlRoot link
		url: 'http://fonts.googleapis.com/css?family=Tangerine:400,700|Droid+Sans+Mono|Droid+Sans';
		type: 'text/css';
		beStylesheet.
	anHtmlRoot stylesheet url: (PTFileLibrary urlOf: #mainCss).
	anHtmlRoot title: 'Pfad-Trainer'! !

!PTRootComponent methodsFor: 'version' stamp: 'MathiasKegelmann 12/15/2011 21:55'!
versionDate
	^ '2011-12-15'! !

!PTRootComponent methodsFor: 'version' stamp: 'MathiasKegelmann 12/14/2011 21:11'!
versionNumber
	^ '0.5'! !

Object subclass: #PTAbstractProblemGenerator
	instanceVariableNames: 'fileSystem source target counter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MkPathTrainer'!

!PTAbstractProblemGenerator commentStamp: 'MathiasKegelmann 12/9/2011 17:49' prior: 0!
I contain the common infrastructure for problem generators!

!PTAbstractProblemGenerator class methodsFor: 'instance creation' stamp: 'MathiasKegelmann 12/9/2011 17:50'!
for: aFileSystem
	^ self new fileSystem: aFileSystem; nextProblem
! !

!PTAbstractProblemGenerator methodsFor: 'accessing' stamp: 'MathiasKegelmann 12/9/2011 17:49'!
count: aBoolean
	counter add: aBoolean ! !

!PTAbstractProblemGenerator methodsFor: 'initialization' stamp: 'MathiasKegelmann 12/9/2011 17:52'!
createCounter
	self subclassResponsibility ! !

!PTAbstractProblemGenerator methodsFor: 'initialization' stamp: 'MathiasKegelmann 12/9/2011 17:50'!
fileSystem: anObject
	fileSystem := anObject! !

!PTAbstractProblemGenerator methodsFor: 'initialization' stamp: 'MathiasKegelmann 12/9/2011 17:51'!
initialize
	super initialize.
	counter := self createCounter! !

!PTAbstractProblemGenerator methodsFor: 'accessing' stamp: 'MathiasKegelmann 12/9/2011 17:50'!
isSufficient
	"have we answered enough questions"
	^ counter value! !

!PTAbstractProblemGenerator methodsFor: 'accessing' stamp: 'MathiasKegelmann 12/9/2011 17:49'!
nextProblem
	self subclassResponsibility! !

!PTAbstractProblemGenerator methodsFor: 'accessing' stamp: 'MathiasKegelmann 12/9/2011 21:54'!
path: aPath on: html
	html span
		class: #path;
		with: aPath! !

!PTAbstractProblemGenerator methodsFor: 'accessing' stamp: 'MathiasKegelmann 12/9/2011 22:00'!
renderProblemOn: html
	html text: 'Was ist der relative Pfad von '.
	self path: source on: html.
	html text: ' nach '.
	self path: target on: html.
	html text: '?'! !

!PTAbstractProblemGenerator methodsFor: 'accessing' stamp: 'MathiasKegelmann 12/9/2011 17:50'!
source
	^ source! !

!PTAbstractProblemGenerator methodsFor: 'accessing' stamp: 'MathiasKegelmann 12/9/2011 17:50'!
target
	^ target! !

PTAbstractProblemGenerator subclass: #PTBranchingProblemGenerator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MkPathTrainer'!

!PTBranchingProblemGenerator methodsFor: 'initialization' stamp: 'MathiasKegelmann 12/9/2011 17:52'!
createCounter
	^ PTSlidingWindowCounter new: 7 allowedErrors: 2! !

!PTBranchingProblemGenerator methodsFor: 'accessing' stamp: 'MathiasKegelmann 12/9/2011 17:54'!
nextProblem
	"generate the next problem"
	| paths |
	paths := fileSystem allPaths.
	source := paths atRandom.
	target := paths atRandom ! !

PTAbstractProblemGenerator subclass: #PTFileReferencingProblemGenerator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MkPathTrainer'!

!PTFileReferencingProblemGenerator methodsFor: 'initialization' stamp: 'MathiasKegelmann 12/9/2011 17:56'!
createCounter
	^ PTSlidingWindowCounter new: 7 allowedErrors: 2! !

!PTFileReferencingProblemGenerator methodsFor: 'accessing' stamp: 'MathiasKegelmann 12/9/2011 18:30'!
nextProblem
	"generate the next problem"
	| paths path |
	paths := fileSystem allPaths copyWithoutAll: #('/' '/html' '/html/ilvesheimmlk').
	source := paths atRandom.
	path := paths atRandom.
	target := 	
		3 atRandom = 1 
			ifTrue: [ path, '/index.html' ]
			ifFalse: [ self targetFileIn: path ]! !

!PTFileReferencingProblemGenerator methodsFor: 'accessing' stamp: 'MathiasKegelmann 12/9/2011 22:08'!
renderProblemOn: html
	html text: 'Wie referenziert man aus der HTML-Datei '.
	self path: source, '/index.html' on: html.
	html text: ' die Datei '.
	self path: target on: html.
	html text: '?'! !

!PTFileReferencingProblemGenerator methodsFor: 'accessing' stamp: 'MathiasKegelmann 12/9/2011 18:08'!
targetFileIn: aDirectoryPath
	"the path must not be /"
	| dir |
	dir := (aDirectoryPath findTokens: $/) last asLowercase.
	^ aDirectoryPath, 
		'/',
		(dir = 'bilder' ifTrue: [ 'ausflug.jpg'] ifFalse: [ dir, '.html' ])! !

PTAbstractProblemGenerator subclass: #PTNonBranchingProblemGenerator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MkPathTrainer'!

!PTNonBranchingProblemGenerator methodsFor: 'initialization' stamp: 'MathiasKegelmann 12/9/2011 17:51'!
createCounter
	^ PTSlidingWindowCounter new: 4 allowedErrors: 1! !

!PTNonBranchingProblemGenerator methodsFor: 'accessing' stamp: 'MathiasKegelmann 12/9/2011 09:21'!
nextProblem
	"generate the next problem"
	| p |
	p := fileSystem maximalPathSliceAtRandom allPaths shuffled copyFrom: 1 to: 2.
	source := p first.
	target := p second! !

Object subclass: #PTFileSystem
	instanceVariableNames: 'directories'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MkPathTrainer'!

!PTFileSystem commentStamp: 'MathiasKegelmann 12/4/2011 22:26' prior: 0!
I simulate a file system of dirctories in a tree structure.

Internally the stucture is stored as nested directory of directories. It can be initialized ba adding concrete paths. They and and all implied parents are automatically added.!

!PTFileSystem class methodsFor: 'instance creation' stamp: 'MathiasKegelmann 12/5/2011 22:52'!
with: aPath
	| fs |
	fs := self new.
	fs addPath: aPath.
	^ fs! !

!PTFileSystem class methodsFor: 'instance creation' stamp: 'MathiasKegelmann 12/6/2011 20:12'!
withAll: paths
	| fs |
	fs := self new.
	paths do: [ :p | fs addPath: p ].
	^ fs! !

!PTFileSystem methodsFor: 'initialize-release' stamp: 'MathiasKegelmann 12/6/2011 07:59'!
addPath: aByteString
	self ensureDirSequence: (self pathAsDirSequence: aByteString) in: directories.
! !

!PTFileSystem methodsFor: 'accessing' stamp: 'MathiasKegelmann 12/6/2011 20:04'!
allPaths
	^ (self allPathsIn: directories) 
		collect: [ :x | x isEmpty ifTrue: [ '/' ] ifFalse: [ x ] ]! !

!PTFileSystem methodsFor: 'private' stamp: 'MathiasKegelmann 12/6/2011 20:54'!
allPathsIn: aDictionary
	| childPaths |
	childPaths := aDictionary associations inject: #() into: [ :result :assoc |
		result, ((self allPathsIn: assoc value) collect: [ :path | '/', assoc key, path ])
	].
	^ childPaths copyWith: ''! !

!PTFileSystem methodsFor: 'testing' stamp: 'MathiasKegelmann 12/4/2011 22:33'!
contains: aPath
	aPath = '/' ifTrue: [ ^ true ]! !

!PTFileSystem methodsFor: 'private' stamp: 'MathiasKegelmann 12/4/2011 22:29'!
directories: aDictionaryOfDictionaries
	directories := aDictionaryOfDictionaries! !

!PTFileSystem methodsFor: 'private' stamp: 'MathiasKegelmann 12/6/2011 08:19'!
ensureDirSequence: anOrderedCollection in: aDictionary
	anOrderedCollection ifEmpty: [ ^self ].
	self 
		ensureDirSequence: (anOrderedCollection allButFirst) 
		in: (aDictionary at: anOrderedCollection first ifAbsentPut: [ Dictionary new ])
! !

!PTFileSystem methodsFor: 'initialize-release' stamp: 'MathiasKegelmann 12/6/2011 08:01'!
initialize
	super initialize.
	directories := Dictionary new! !

!PTFileSystem methodsFor: 'copying' stamp: 'MathiasKegelmann 12/6/2011 21:24'!
maximalPathSliceAtRandom
	^ PTFileSystem with: self maximalPaths atRandom ! !

!PTFileSystem methodsFor: 'accessing' stamp: 'MathiasKegelmann 12/6/2011 21:19'!
maximalPaths
	| allPaths |
	allPaths := self allPaths.
	^ allPaths select: [ :p | allPaths allSatisfy: [ :q | q = p or: [ (q beginsWith: p) not ] ] ]! !

!PTFileSystem methodsFor: 'private' stamp: 'MathiasKegelmann 12/5/2011 22:50'!
pathAsDirSequence: aByteString
	^ aByteString findTokens: $/! !

Object subclass: #PTRelativePathFinder
	instanceVariableNames: 'to from'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MkPathTrainer'!

!PTRelativePathFinder class methodsFor: 'instance creation' stamp: 'MathiasKegelmann 12/14/2011 20:24'!
from: aByteString to: aByteString2 
	^ self new from: aByteString  to: aByteString2! !

!PTRelativePathFinder methodsFor: 'finding' stamp: 'MathiasKegelmann 12/15/2011 22:15'!
findAncestorThenDown
	| walker n up |
	walker := PTRelativePathWalker startingAt: from.
	n := 0.
	[ to beginsWith: walker currentPath ] whileFalse: [
		n := n + 1.
		walker up.
	].
	up := OrderedCollection streamContents: [ :s | n timesRepeat: [ s nextPut: '..'] ].
	^ up copyWithAll: (self findStepsDown: walker)! !

!PTRelativePathFinder methodsFor: 'finding' stamp: 'MathiasKegelmann 12/15/2011 22:08'!
findPath
	^ String streamContents: [ :stream | self findSteps asStringOn: stream delimiter: '/' ]! !

!PTRelativePathFinder methodsFor: 'finding' stamp: 'MathiasKegelmann 12/15/2011 22:11'!
findSteps
	from = to ifTrue: [ ^ #('.') ].
	^ self findAncestorThenDown! !

!PTRelativePathFinder methodsFor: 'finding' stamp: 'MathiasKegelmann 12/15/2011 22:14'!
findStepsDown: walker
	| steps |
	steps := OrderedCollection new.
	[ walker currentPath = to ]
		whileFalse: [ 
			| next |
			next := ((to allButFirst: walker currentPath size) findTokens: $/) first.
			steps add: next.
			walker step: next ].
	^ steps! !

!PTRelativePathFinder methodsFor: 'initializing' stamp: 'MathiasKegelmann 12/14/2011 20:12'!
from: aByteString to: aByteString2 
	from := aByteString.
	to := aByteString2 ! !

Object subclass: #PTRelativePathWalker
	instanceVariableNames: 'currentPath'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MkPathTrainer'!

!PTRelativePathWalker commentStamp: 'MathiasKegelmann 12/7/2011 07:58' prior: 0!
I compute relative paths!

!PTRelativePathWalker class methodsFor: 'instance creation' stamp: 'MathiasKegelmann 12/7/2011 08:02'!
startingAt: aByteString 
	| pw fw |
	pw := self new.
	pw currentPath: aByteString.
	^ pw! !

!PTRelativePathWalker methodsFor: 'updating' stamp: 'MathiasKegelmann 12/15/2011 22:33'!
add: aByteString
	"add a  step to the path, even if it is . or .."
	currentPath add: aByteString! !

!PTRelativePathWalker methodsFor: 'accessing' stamp: 'MathiasKegelmann 12/15/2011 22:26'!
currentPath
	^ '/', self currentPathWithoutLeadingSlash ! !

!PTRelativePathWalker methodsFor: 'accessing' stamp: 'MathiasKegelmann 12/7/2011 08:05'!
currentPath: aByteString 
	currentPath := aByteString findTokens: $/! !

!PTRelativePathWalker methodsFor: 'accessing' stamp: 'MathiasKegelmann 12/15/2011 22:26'!
currentPathWithoutLeadingSlash
	^ String streamContents: [ :stream | currentPath asStringOn: stream delimiter: '/' ]! !

!PTRelativePathWalker methodsFor: 'initialize-release' stamp: 'MathiasKegelmann 12/7/2011 08:04'!
initialize
	currentPath := OrderedCollection new! !

!PTRelativePathWalker methodsFor: 'updating' stamp: 'MathiasKegelmann 12/15/2011 22:32'!
step: aByteString
	aByteString = '..'
		ifTrue: [ ^ self up ].
	aByteString = '.'
		ifFalse: [ self add: aByteString ]! !

!PTRelativePathWalker methodsFor: 'updating' stamp: 'MathiasKegelmann 12/7/2011 18:31'!
up
	currentPath removeLast.! !

!PTRelativePathWalker methodsFor: 'updating' stamp: 'MathiasKegelmann 12/7/2011 18:33'!
walkPath: aByteString
	(aByteString beginsWith: '/')
		ifTrue: [ Error signal: 'Absoulte path' ].
	(aByteString findTokens: $/) do: [ :step | self step: step ]! !

Object subclass: #PTSlidingWindowCounter
	instanceVariableNames: 'windowSize allowedErrors answers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MkPathTrainer'!

!PTSlidingWindowCounter commentStamp: 'MathiasKegelmann 12/9/2011 07:48' prior: 0!
I count the number of "true" elements added among the last n ones.!

!PTSlidingWindowCounter class methodsFor: 'instance creation' stamp: 'MathiasKegelmann 12/9/2011 07:56'!
new: size allowedErrors: errors
	^ self new windowSize: size; allowedErrors: errors ! !

!PTSlidingWindowCounter methodsFor: 'adding' stamp: 'MathiasKegelmann 12/9/2011 08:14'!
add: aBoolean 
	answers add: aBoolean.
	answers size > windowSize ifTrue: [ answers := answers allButFirst ]! !

!PTSlidingWindowCounter methodsFor: 'initialization' stamp: 'MathiasKegelmann 12/9/2011 07:55'!
allowedErrors: anObject
	allowedErrors := anObject! !

!PTSlidingWindowCounter methodsFor: 'initialization' stamp: 'MathiasKegelmann 12/9/2011 08:00'!
initialize
	answers := OrderedCollection new! !

!PTSlidingWindowCounter methodsFor: 'accessing' stamp: 'MathiasKegelmann 12/9/2011 08:12'!
value
	"are there at least n answers, the last one correct and at most m errors"
	^ answers size = windowSize 
		and: [ answers last and: 
			[ (answers select: #not) size <= allowedErrors ] ]! !

!PTSlidingWindowCounter methodsFor: 'initialization' stamp: 'MathiasKegelmann 12/9/2011 08:00'!
windowSize: n
	windowSize := n! !
PTRootComponent initialize!
